@@ -1,5 +1,6 @@
 import itertools
 import logging
+import math
 import os
 import struct
 import weakref
@@ -196,10 +197,10 @@ class Base:
                 tuple(a for a in annotations if not a.eliminatable and not a.relocatable)
             ))
 
-            relocatable_annotations = OrderedDict((e, True) for e in tuple(itertools.chain(
+            relocatable_annotations = tuple(OrderedDict((e, True) for e in tuple(itertools.chain(
                 itertools.chain.from_iterable(a._relocatable_annotations for a in ast_args) if not skip_child_annotations else tuple(),
                 tuple(a for a in annotations if not a.eliminatable and a.relocatable)
-            ))).keys()
+            ))).keys())
 
             annotations = tuple(itertools.chain(
                 itertools.chain.from_iterable(a._relocatable_annotations for a in ast_args) if not skip_child_annotations else tuple(),
@@ -212,7 +213,12 @@ class Base:
         if hash is not None:
             h = hash
         elif op in {'BVS', 'BVV', 'BoolS', 'BoolV', 'FPS', 'FPV'} and not annotations:
-            h = (op, kwargs.get('length', None), a_args)
+            if op == "FPV" and a_args[0] == 0.0 and math.copysign(1, a_args[0]) < 0:
+                # Python does not distinguish between +0.0 and -0.0 so we add sign to tuple to distinguish
+                h = (op, kwargs.get('length', None), ("-",) + a_args)
+            else:
+                h = (op, kwargs.get('length', None), a_args)
+
             cache = cls._leaf_cache
         else:
             h = Base._calc_hash(op, a_args, kwargs) if hash is None else hash
@@ -427,8 +433,12 @@ class Base:
         if simplified is not None:
             op = simplified.op
 
-        all_operations = operations.leaf_operations_symbolic | {'union'}
-        if 'annotations' not in kwargs: kwargs['annotations'] = self.annotations
+        all_operations = operations.leaf_operations_symbolic_with_union
+        if 'annotations' not in kwargs:
+            # special case: if self is one of the args, we do not copy annotations over from self since child
+            # annotations will be re-processed during AST creation.
+            if not args or not any(self is arg for arg in args):
+                kwargs['annotations'] = self.annotations
         if 'variables' not in kwargs and op in all_operations: kwargs['variables'] = self.variables
         if 'uninitialized' not in kwargs: kwargs['uninitialized'] = self._uninitialized
         if 'symbolic' not in kwargs and op in all_operations: kwargs['symbolic'] = self.symbolic
