@@ -144,6 +144,9 @@ def extract_check(high, low, bv):
 
     return True, ""
 
+def extend_check(amount, value):
+    return amount >= 0, "Extension length must be nonnegative"
+
 def concat_length_calc(*args):
     return sum(arg.length for arg in args)
 
@@ -364,6 +367,7 @@ inverse_operations = {
 leaf_operations = backend_symbol_creation_operations | backend_creation_operations
 leaf_operations_concrete = backend_creation_operations
 leaf_operations_symbolic = backend_symbol_creation_operations
+leaf_operations_symbolic_with_union = leaf_operations_symbolic | {'union'}
 
 #
 # Reversibility
