@@ -32,7 +32,6 @@ class SimplificationManager:
             'SignExt': self.signext_simplifier,
             'fpToIEEEBV': self.fptobv_simplifier,
             'fpToFP': self.fptofp_simplifier,
-            'StrExtract': self.str_extract_simplifier,
             'StrReverse': self.str_reverse_simplifier,
         }
 
@@ -171,6 +170,9 @@ class SimplificationManager:
     def lshift_simplifier(val, shift):
         if (shift == 0).is_true():
             return val
+        if val.op == '__lshift__':
+            real_val, inner_shift = val.args
+            return real_val << (inner_shift + shift)
 
     @staticmethod
     def eq_simplifier(a, b):
@@ -471,6 +473,9 @@ class SimplificationManager:
         if filter_func: new_args = filter_func(new_args)
         if not new_args and 'initial_value' in kwargs:
             return kwargs['initial_value']
+        # if a single arg is left, don't create an op for it
+        if len(new_args) == 1:
+            return new_args[0]
         return next(a for a in args if isinstance(a, ast.Base)).make_like(op_name, new_args,
                                                                           variables=variables,
                                                                           simplify=False)
@@ -624,6 +629,10 @@ class SimplificationManager:
                 return a
             elif a is b:
                 return a
+            elif (a == 0).is_true():
+                return ast.all_operations.BVV(0, a.size())
+            elif (b == 0).is_true():
+                return ast.all_operations.BVV(0, a.size())
             elif a.op == "Concat" and len(a.args) == 2:
                 # Concat(a.args[0], a.args[1]) & b  ==>  ZeroExt(size, a.args[1])
                 # maybe we can drop the second argument
@@ -845,18 +854,6 @@ class SimplificationManager:
         expr = (masked_a << lshift_) | (masked_a >> rshift_)
         return expr
 
-    @staticmethod
-    def str_extract_simplifier(start_idx, count, val):
-        if start_idx == 0 and count == val.string_length:
-            return val
-        # if we are dealing with a chain of extractions on the same string we can
-        # simplify the chain in one single StrExtract
-        if val.op == 'StrExtract':
-            v_start_idx, _, v_str = val.args
-            new_start = v_start_idx + start_idx
-            new_count = count
-            return v_str.StrExtract(new_start, new_count, v_str)
-
     @staticmethod
     def str_reverse_simplifier(arg):
         return arg
@@ -884,7 +881,7 @@ class SimplificationManager:
                     v = v >> 1
                     zero_bits -= 1
                 if zero_bits > 0:
-                    # the higher `zero_bits` bits are zero
+                    # the high `zero_bits` bits are zero
                     # check the constant
                     b_higher = b[b.size() - 1 : b.size() - zero_bits]
                     b_higher_bits_are_0: Optional[bool] = None
@@ -895,13 +892,25 @@ class SimplificationManager:
 
                     if b_higher_bits_are_0 is True:
                         # extra check: can we get rid of the mask
-                        b_lower = b[b.size() - 1 - zero_bits: 0]
+                        if zero_bits == b.size():
+                            # the mask is 0, which means the left-hand side becomes 0 after masking
+                            return op(ast.all_operations.BVV(0, b.size()), b)
+
+                        b_highbit_idx = b.size() - 1 - zero_bits
+                        if b.size() % 8 == 0:
+                            # originally, b was 8-bit aligned. Can we keep the size of the new expression 8-byte aligned?
+                            if (b_highbit_idx + 1) % 8 != 0:
+                                b_highbit_idx += 8 - (b_highbit_idx + 1) % 8
+                        b_lower = b[b_highbit_idx : 0]
                         if mask_allones:
                             # yes!
-                            return op(a_arg0[b.size() - 1 - zero_bits : 0], b_lower)
+                            return op(a_arg0[b_highbit_idx : 0], b_lower)
                         else:
                             # nope
-                            return op(a_arg0[b.size() - 1 - zero_bits: 0] & a_arg1.args[0], b_lower)
+                            if b_highbit_idx == b.size() - 1:
+                                # no change is happening
+                                return None
+                            return op(a_arg0[b_highbit_idx : 0] & a_arg1.args[0], b_lower)
                     elif b_higher_bits_are_0 is False:
                         return ast.all_operations.false if op is operator.__eq__ else ast.all_operations.true
 
